package de.hhu.phil.smt.lm;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Scanner;

import de.hhu.phil.smt.SMT;
import de.hhu.phil.smt.misc.Pair;

/*
 * This class represents a bigram language model.
 * 
 * All missing pieces are marked with TODO
 */
public class BigramModel {
	
	// data structure for the vocabulary
	private HashSet<String> vocabulary;
	
	// data structures to hold counts for unigrams and bigrams
	private HashMap<String, Integer> uniCount;
	private HashMap<Pair<String, String>, Integer> biCount;
	
	/*
	 * Constructor reads the training data file which contains one
	 * sentence per line (that is tokenized by whitespace) and
	 * counts unigrams and bigrams.
	 */
	public BigramModel(String dataFile) throws FileNotFoundException {
		vocabulary = new HashSet<String>();
		uniCount = new HashMap<String, Integer>();
		biCount = new HashMap<Pair<String, String>, Integer>();
		
		// determine the vocabulary
		determineVocabulary(dataFile, 1);
		
		// read in the training data und fill the data structures
		// uniCount and biCount
		Scanner input = new Scanner(new File(dataFile));
		String line = "";
		int c = 0;
		while (input.hasNextLine()) {
			line = input.nextLine();
			c++;
			addToCounts(line);
		}
		
		input.close();
		System.out.println("The training data has " + c + " lines.");
	}
	
	/*
	 * @Param dataFile file with the training data
	 * @Param threshold frequency threshold
	 * Determines the vocabulary given the training data. 
	 * Only words that occur more frequently than the threshold are
	 * part of the vocabulary.
	 */
	private void determineVocabulary(String dataFile, int threshold) throws FileNotFoundException {
		// data structure to count word frequencies
		HashMap<String, Integer> counts = new HashMap<String, Integer>();
		// read in the training data file line by line
		Scanner input = new Scanner(new File(dataFile));
		String line = "";
		while (input.hasNextLine()) {
			line = input.nextLine();
			// tokenize the sentence
			String[] tokens = SMT.tokenizeWS(line);
			// fill the data structure
			for (String tok : tokens) {
				Integer c = counts.get(tok);
				if (c == null) {
					c = 0;
				}
				counts.put(tok, c+1);
			}
		}
		input.close();
		
		// add words which we have seen more often than threshold to the vocabulary
		for (Map.Entry<String, Integer> entry : counts.entrySet()) {
			if (entry.getValue() > threshold) {
				vocabulary.add(entry.getKey());
			}
		}
		
		// also add start-of-sentence and end-of-sentence token to the vocabulary
		vocabulary.add(SMT.START_TOK);
		vocabulary.add(SMT.END_TOK);
	}
	
	// fill the uniCount and biCount data structures
	private void addToCounts(String line) {
		String lineWithStartEnd = SMT.addStartEnd(line);
		String[] tokens = SMT.tokenizeWS(lineWithStartEnd);
		
		// TODO
			
	}

	// Calculate the probability P(w2|w1) using MLE and return it
	public double getProb(String w1, String w2) {
		// TODO
		return -1000;
	}
	
	// Calculate the probability P(w2|w1) using MLE and add-one smoothing,
	// and return it
	public double getProbAddOneSmooth(String w1, String w2) {
		// TODO
		return -1000;
	}

	// Calculate the probability that your bigram language model assigns
	// to the test sentence. Use the smoothed estimates.
	public double score(String test) {
		String testWithStartEnd = SMT.addStartEnd(test);
		String[] tokens = SMT.tokenizeWS(testWithStartEnd);

		// TODO
		
		return -1000;
	}
	
	

}
